@@ -1,5 +1,3 @@
-import sys
-
 from typing import List
 from typing import Union
 
@@ -7,7 +5,6 @@ from poetry.io import NullIO
 from poetry.packages import Dependency
 from poetry.packages import Locker
 from poetry.packages import Package
-from poetry.packages.constraints import parse_constraint as parse_generic_constraint
 from poetry.puzzle import Solver
 from poetry.puzzle.operations import Install
 from poetry.puzzle.operations import Uninstall
@@ -17,7 +14,6 @@ from poetry.repositories import Pool
 from poetry.repositories import Repository
 from poetry.repositories.installed_repository import InstalledRepository
 from poetry.semver import parse_constraint
-from poetry.semver import Version
 from poetry.utils.helpers import canonicalize_name
 
 from .base_installer import BaseInstaller
@@ -47,6 +43,7 @@ class Installer:
         self._dev_mode = True
         self._develop = []
         self._execute_operations = True
+        self._lock = False
 
         self._whitelist = []
 
@@ -111,6 +108,16 @@ class Installer:
 
         return self
 
+    def lock(self):  # type: () -> Installer
+        """
+        Prepare the installer for locking only.
+        """
+        self.update()
+        self.execute_operations(False)
+        self._lock = True
+
+        return self
+
     def is_updating(self):  # type: () -> bool
         return self._update
 
@@ -132,7 +139,7 @@ class Installer:
     def _do_install(self, local_repo):
         locked_repository = Repository()
         if self._update:
-            if self._locker.is_locked():
+            if self._locker.is_locked() and not self._lock:
                 locked_repository = self._locker.locked_repository(True)
 
                 # If no packages have been whitelisted (The ones we want to update),
@@ -180,7 +187,14 @@ class Installer:
             # currently installed
             ops = self._get_operations_from_lock(locked_repository)
 
-        self._populate_local_repo(local_repo, ops, locked_repository)
+        self._populate_local_repo(local_repo, ops)
+
+        if self._update:
+            self._write_lock_file(local_repo)
+
+            if self._lock:
+                # If we are only in lock mode, no need to go any further
+                return 0
 
         root = self._package
         if not self.is_dev_mode():
@@ -272,20 +286,18 @@ class Installer:
                 )
             )
 
-        # Writing lock before installing
+        self._io.writeln("")
+        for op in ops:
+            self._execute(op)
+
+    def _write_lock_file(self, repo):  # type: (Repository) -> None
         if self._update and self._write_lock:
-            updated_lock = self._locker.set_lock_data(
-                self._package, local_repo.packages
-            )
+            updated_lock = self._locker.set_lock_data(self._package, repo.packages)
 
             if updated_lock:
                 self._io.writeln("")
                 self._io.writeln("<info>Writing lock file</>")
 
-        self._io.writeln("")
-        for op in ops:
-            self._execute(op)
-
     def _execute(self, operation):  # type: (Operation) -> None
         """
         Execute a given operation.
@@ -374,38 +386,17 @@ class Installer:
 
         self._installer.remove(operation.package)
 
-    def _populate_local_repo(self, local_repo, ops, locked_repository):
-        # We walk through all operations and add/remove/update accordingly
+    def _populate_local_repo(self, local_repo, ops):
         for op in ops:
-            if isinstance(op, Update):
+            if isinstance(op, Uninstall):
+                continue
+            elif isinstance(op, Update):
                 package = op.target_package
             else:
                 package = op.package
 
-            acted_on = False
-            for pkg in locked_repository.packages:
-                if pkg.name == package.name:
-                    # The package we operate on is in the local repo
-                    if op.job_type == "update":
-                        if pkg.version == package.version:
-                            break
-
-                        local_repo.remove_package(pkg)
-                        local_repo.add_package(op.target_package)
-                    elif op.job_type == "uninstall":
-                        local_repo.remove_package(op.package)
-                    else:
-                        # Even though the package already exists
-                        # in the lock file we will prefer the new one
-                        # to force updates
-                        local_repo.remove_package(pkg)
-                        local_repo.add_package(package)
-
-                    acted_on = True
-
-            if not acted_on:
-                if not local_repo.has_package(package):
-                    local_repo.add_package(package)
+            if not local_repo.has_package(package):
+                local_repo.add_package(package)
 
     def _get_operations_from_lock(
         self, locked_repository  # type: Repository
